/*-----------------------------------------------------------------------------+
|       File:       seriell_schnell.c                                         |
|       Autor:      Bubbers                                                    |
|       Klasse:     TGJ1                                                       |
|       Controller: AT89C5131 (Atmel)                                          | 
|       Datum:      16.11.10                                                   |
|       Version:    1.0                                                        |
+------------------------------------------------------------------------------+
|  Beschreibung:                                                               |	
|    Controller schickt bei Druck auf den Tasters P3_2 das mit den 8 Schaltern |
|    an P1 eingestellte Byte ber die serielle Schnittstelle    | 
|    Dieses Zeichen wird auch an P0 mit 8 LEDs dargestellt.                    |
|    Ein empfangenes Byte wird an die 8 LEDs von P2 ausgegeben                 |
+-----------------------------------------------------------------------------*/

#include <at89c5131.h>           // Registeradressen fuer Atmel AT89C5131
 
sfr  at P2   AnzeigeEmpfang;     // 8 LEDs 
sfr  at P0   AnzeigeSenden; 
sfr  at P1   Sendebyte;
sbit at P3_3 Sendetaster;        // Blauer Taster    = Byte an 8 Schaltern senden

                                  // Reloadwert des Baudratengenerators  fr 9600 und keinem X2-Modus
                                  // BRL = 256- ( 2^SMOD1 * Fper) / ( 6^(1-SPD) * 16 * Baudrate) 
										    // Fper = Fquarz/2 = 6MHz wenn kein x2-Mode (bei X2-Mode Fper = 12 MHz)
#define reloadBRL 250             // BRL = 256 - (12000000)/(16*38400) = 236 ok
											 // BRL = 256 - (12000000)/(16*57600) = 243 ok
                                  // BRL = 256 - (12000000)/(16*128000) = 250 (ungenau)							    


void entprellung (void) //----------- Zeitverzgerung zur Entprellung -----------------------------------
{  unsigned char c;
   for (c=255;c!=0;c--);
}
void main (void) //------------------ Hauptprogramm -----------------------------------------------------
{
   AnzeigeEmpfang = 0;                 // Anzeigen aus
   AnzeigeSenden = 0;                  //
                                       // Initialisierung serielle Schnittstelle
//Initialisierung fuer 250000 Bit/s beim Senden (Baudratengenerator) und xxx Bit/s beim Empfangen (Timer2)
	CKCON0= 0b00000001;		// X2 Modus (6 Takte pro Befehl, Statt wie normal 12)
	PCON=PCON | 0b10000000; // SMOD1=1
	BDRCON= 0b00011111;		// int.Baudraten-Generator nur fr Senden und Empfangen TBCK=1, RBCK=1, SPD=1, Generator starten
	BRL=reloadBRL; 			// Reloadwert fr int.. Baudratengenerator
	SCON= 0b01010000; 		// 8-Bit-UART, Empfang zulassen
	ES=0;							// keiner serieller Interput  
	TI=1;							// Sendeflag setzen              
	RI=0;                   // Empfangsflag lschen
	ES=0;                   // kein serieller Interrupt
                                      
 	while (1)                           // Endlosschleife Hauptprogramm
 	{
 		if (RI)                          // Wenn Zeichen empfangen
 			{AnzeigeEmpfang = SBUF;       // abholen und anzeigen
 			RI = 0;                       // und fr neuen Empfang vorbereiten (Empfangsflag lschen)
 			}
 		else if (Sendetaster)            // Wenn Sendertaster gedrckt
 				{while (!TI);              // warten falls altes Byte noch nicht fertig gesendet
 				AnzeigeSenden = Sendebyte; // Sendebyte dual anzeigen
 				SBUF = Sendebyte;          // und senden
 				TI = 0;                    // Sendeflag lschen
 				entprellung();
 				while (Sendetaster);       // warten bis Taster losgelassen
 				entprellung();
 				} // Ende Senden

	} // Ende Hauptprogrammschleife
} // Ende Hauptprogramm	  	

